from urllib.request import urlopen
import pandas as pd
from bs4 import BeautifulSoup

url = 'http://bvmf.bmfbovespa.com.br/cias-listadas/empresas-listadas/ResumoEventosCorporativos.aspx?tab=3&codigoCvm='


def busca_eventos_acoes(codigo_cvm):
    with urlopen(url + codigo_cvm) as html:
        content = html.read().decode('utf-8')
        return _eventos_em_ativos(content) + _dividendos(content)


def _eventos_em_ativos(html):
    soup = BeautifulSoup(html, 'html.parser')
    table = soup.find('table', {'id': 'ctl00_contentPlaceHolderConteudo_grdBonificacao_ctl01'})
    try:
        df = pd.read_html(str(table), decimal='.', thousands=',')[0]
    except ValueError:
        return []
    df.drop(columns=['Deliberado em',
                     'Observações'], inplace=True)
    df = df.rename(columns={'Proventos': 'tipo',
                            'Código ISIN': 'isin',
                            'Negócios com até': 'data_com',
                            'Ativo Emitido': 'isin_destino',
                            '% / Fator de Grupamento': 'valor'})
    df['data_com'] = pd.to_datetime(df['data_com'], dayfirst=True, errors='coerce')
    return df.to_dict('records')


def _dividendos(html):
    soup = BeautifulSoup(html, 'html.parser')
    table = soup.find('table', {'id': 'ctl00_contentPlaceHolderConteudo_grdDividendo_ctl01'})
    try:
        df = pd.read_html(str(table), decimal=',', thousands='.')[0]
    except ValueError:
        return []
    df.drop(columns=['Deliberado em',
                     'Relativo a',
                     'Observações'], inplace=True)
    df = df.rename(columns={'Proventos': 'tipo',
                            'Código ISIN': 'isin',
                            'Negócios com até': 'data_com',
                            'Valor (R$)': 'valor',
                            'Início de Pagamento': 'data_pgto'})
    df['data_com'] = pd.to_datetime(df['data_com'], dayfirst=True, errors='coerce')
    df['data_pgto'] = pd.to_datetime(df['data_pgto'], dayfirst=True, errors='coerce')
    return df.to_dict('records')





from .codigos_cvm import busca_codigos_cvm
from .codigos_negociacao import busca_codigos_negociacao
from .eventos_acoes import busca_eventos_acoes

from .fii_listados import busca_codigos_fii
from .eventos_fii import busca_eventos_fii

from .negociacao import busca_trades

from .proventos_fii import busca_proventos_no_range

from tqdm import tqdm
from concurrent import futures


def busca_dados_acoes():
    codigos = busca_codigos_cvm()
    with futures.ThreadPoolExecutor(max_workers=8) as executor:
        return list(tqdm(executor.map(_busca_dados_codigo_cvm, codigos),
                         total=len(codigos),
                         desc='Buscando códigos negociação'))


def busca_dados_fii():
    fiis = busca_codigos_fii()
    with futures.ThreadPoolExecutor(max_workers=8) as executor:
        return list(tqdm(executor.map(_busca_dados_codigo_fii, fiis),
                         total=len(fiis),
                         desc='Buscando informações dos FIIs'))


def _busca_dados_codigo_cvm(codigo):
    cod_negs = busca_codigos_negociacao(codigo['codigo_cvm'])
    return {
        'codigo_cvm': codigo['codigo_cvm'],
        'nome': codigo['nome'],
        'symbols': cod_negs['symbols'],
        'isin': cod_negs['isin'],
        'eventos': busca_eventos_acoes(codigo['codigo_cvm'])
    }


def _busca_dados_codigo_fii(fii):
    return {
        'sigla': fii['sigla'],
        'nome': fii['nome'],
        'symbols': [fii['sigla'] + '11'],
        'eventos': busca_eventos_fii(fii['sigla'])
    }



from urllib.request import urlopen
from bs4 import BeautifulSoup
import re
import pandas as pd
from more_itertools import unique_everseen
from tqdm import tqdm

url = 'http://bvmf.bmfbovespa.com.br/cias-listadas/empresas-listadas/BuscaEmpresaListada.aspx?Letra='
alphanum = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'


def busca_codigos_cvm():
    dados = []
    for letra in tqdm(alphanum, desc='Baixando codigos CVM'):
        with urlopen(url + letra) as html:
            soup = BeautifulSoup(html, 'html.parser')
        table = soup.find('table', {'id': 'ctl00_contentPlaceHolderConteudo_BuscaNomeEmpresa1_grdEmpresa_ctl01'})

        try:
            df = pd.read_html(str(table))[0]
        except ValueError:
            continue

        codigos = list(unique_everseen(
            [link.get('href').split('=')[1] for link in soup.findAll('a', attrs={'href': re.compile("codigoCvm=")})]
        ))
        df['codigo_cvm'] = codigos
        df.drop(columns=['Razão Social',
                         'Segmento'], inplace=True)
        df = df.rename(columns={'Nome de Pregão': 'nome'})
        dados += df.to_dict('records')
    return dados



from urllib.request import urlopen
import re

url = 'http://bvmf.bmfbovespa.com.br/pt-br/mercados/acoes/empresas/ExecutaAcaoConsultaInfoEmp.asp?CodCVM='


def busca_codigos_negociacao(codigo_cvm):
    data = {'codigo_cvm': codigo_cvm}
    with urlopen(url + codigo_cvm) as html:
        source = html.read().decode('utf-8')
        symbols = re.compile("var symbols = '(.*)';").search(source)
        if symbols:
            data['symbols'] = symbols.group(1).split('|')

        isin = re.compile("ISIN:.*\r\n(.*)<br", re.MULTILINE).search(source)
        if isin:
            data['isin'] = [i.strip() for i in isin.group(1).split(',')]
        else:
            print('não encontrou isin para ' + codigo_cvm)

    return data



from urllib.request import urlopen
from bs4 import BeautifulSoup
import pandas as pd

url = 'http://bvmf.bmfbovespa.com.br/Fundos-Listados/FundosListados.aspx?tipoFundo=imobiliario'


def busca_codigos_fii():
    with urlopen(url) as html:
        content = html.read().decode('utf-8')
        soup = BeautifulSoup(content, 'html.parser')

    table = soup.find('table')
    try:
        df = pd.read_html(str(table))[0]
        df.drop(columns=['Fundo',
                         'Segmento'], inplace=True)
        df = df.rename(columns={'Razão Social': 'nome',
                                'Código': 'sigla'})
    except ValueError:
        return []
    return df.to_dict('records')



from setuptools import setup, find_packages


setup(
    name='b3_scrapers',
    version='0.3',
    description='Scrapers da B3',
    url='https://github.com/marcus-almendro/b3_scrapers',
    author='Marcus Almendro',
    author_email='marcus.almendro@gmail.com',
    packages=find_packages(),
    install_requires=['beautifulsoup4', 'pandas', 'lxml', 'html5lib', 'more-itertools', 'selenium', 'tqdm'],
)


import pandas as pd
from bs4 import BeautifulSoup
from selenium.webdriver import Chrome, ChromeOptions
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

BASE_URL = 'https://cei.b3.com.br/'
chrome_options = ChromeOptions()
chrome_options.add_argument('--no-sandbox')
chrome_options.add_argument('--headless')
chrome_options.add_argument('--disable-gpu')
driver = None


def busca_trades(cpf, senha):
    global driver
    try:
        driver = Chrome(options=chrome_options)
        driver.implicitly_wait(20)
        _login(cpf, senha)
        lista = []
        for corretora in _corretoras_disponiveis():
            _abre_consulta_trades(corretora)
            lista += _ler_tabela(corretora)
        return lista
    finally:
        driver.quit()


def _login(cpf, senha):
    driver.get(BASE_URL)
    txt_login = driver.find_element_by_id('ctl00_ContentPlaceHolder1_txtLogin')
    txt_login.clear()
    txt_login.send_keys(cpf)
    txt_senha = driver.find_element_by_id('ctl00_ContentPlaceHolder1_txtSenha')
    txt_senha.clear()
    txt_senha.send_keys(senha)
    btn_logar = driver.find_element_by_id('ctl00_ContentPlaceHolder1_btnLogar')
    btn_logar.click()
    WebDriverWait(driver, 200).until(EC.visibility_of_element_located((By.ID, 'objGrafPosiInv')))


def _corretoras_disponiveis():
    driver.get(BASE_URL + 'negociacao-de-ativos.aspx')
    from selenium.webdriver.support.select import Select
    agentes = Select(driver.find_element_by_id('ctl00_ContentPlaceHolder1_ddlAgentes'))
    first_option = int(agentes.first_selected_option.get_attribute('value'))
    if first_option == -1:  # Selecione
        return list(filter(lambda c: c > -1, map(lambda o: int(o.get_attribute('value')), agentes.options)))
    else:
        return [first_option]


def _abre_consulta_trades(corretora):
    driver.get(BASE_URL + 'negociacao-de-ativos.aspx')

    from selenium.webdriver.support.select import Select
    agentes = Select(driver.find_element_by_id('ctl00_ContentPlaceHolder1_ddlAgentes'))

    if agentes.first_selected_option.text.upper() == 'SELECIONE':
        agentes.select_by_value(str(corretora))
    else:
        btn_consultar = WebDriverWait(driver, 200).until(
            EC.visibility_of_element_located((By.ID, 'ctl00_ContentPlaceHolder1_btnConsultar')))
        btn_consultar.click()

        def not_disabled(drv):
            try:
                drv.find_element_by_id('ctl00_ContentPlaceHolder1_ddlAgentes')
            except NoSuchElementException:
                return False
            return drv.find_element_by_id('ctl00_ContentPlaceHolder1_ddlAgentes').get_attribute(
                "disabled") is None

        WebDriverWait(driver, 200).until(not_disabled)

    btn_consultar = driver.find_element_by_id('ctl00_ContentPlaceHolder1_btnConsultar')
    btn_consultar.click()

    WebDriverWait(driver, 200).until(EC.visibility_of_element_located(
        (By.ID, 'ctl00_ContentPlaceHolder1_rptAgenteBolsa_ctl00_rptContaBolsa_ctl00_pnAtivosNegociados')))


def _ler_tabela(corretora):
    soup = BeautifulSoup(driver.page_source, 'html.parser')

    top_div = soup.find('div', {
        'id': 'ctl00_ContentPlaceHolder1_rptAgenteBolsa_ctl00_rptContaBolsa_ctl00_pnAtivosNegociados'})

    table = top_div.find(lambda tag: tag.name == 'table')

    df = pd.read_html(str(table), decimal=',', thousands='.')[0]

    df = df.dropna(subset=['Mercado'])
    df = df.rename(columns={'Código Negociação': 'papel',
                            'Compra/Venda': 'operacao',
                            'Quantidade': 'qtde',
                            'Data do Negócio': 'data',
                            'Preço (R$)': 'preco'})

    df['data'] = pd.to_datetime(df['data'], dayfirst=True, errors='coerce')
    df['operacao'] = df.apply(lambda row: 'venda' if row.operacao == 'V' else 'compra', axis=1)

    df.drop(columns=['Mercado',
                     'Prazo/Vencimento',
                     'Especificação do Ativo',
                     'Valor Total(R$)',
                     'Fator de Cotação'], inplace=True)

    df['corretora'] = corretora

    return df.to_dict('records')



from urllib.request import urlopen, Request
import pandas as pd
from bs4 import BeautifulSoup
import ssl
from tqdm import tqdm

url = 'https://fnet.bmfbovespa.com.br/fnet/publico/exibirDocumento?id='


def busca_proventos_no_range(inicio, fim):
    proventos = []
    headers = {'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8'}
    for i in tqdm(range(inicio, fim)):
        with urlopen(Request(url + str(i), headers=headers), context=ssl._create_unverified_context()) as html:
            if html.getheader('Content-Type') == 'text/html; charset=UTF-8':
                content = html.read().decode('utf-8')
                provento = _parse_provento(i, content)
                if provento:
                    proventos.append(provento)
            else:
                continue
    return proventos


def _parse_provento(i, html):
    if not html.find('Informações sobre Pagamento de Proventos'):
        return None

    soup = BeautifulSoup(html, 'html.parser')
    try:
        table = soup.findAll('table')[1]
        df = pd.read_html(str(table), decimal=',', thousands='.')[0]
        df.drop(columns=[0, 2], inplace=True)
        df = df.transpose()
        df = df.rename(columns={3: 'data_com', 5: 'valor', 4: 'data_pgto'})
        df['data_com'] = pd.to_datetime(df['data_com'], dayfirst=True, errors='coerce')
        df['data_pgto'] = pd.to_datetime(df['data_pgto'], dayfirst=True, errors='coerce')
        valores = df.to_dict('records')[0]
        spans = soup.findAll('span', {'class', 'dado-cabecalho'})
        return {
            'id': i,
            'codigo_negociacao': spans[-1].getText(),
            'isin': spans[-2].getText(),
            'data_com': valores['data_com'],
            'data_pgto': valores['data_pgto'],
            'valor': valores['valor'],
        }
    except ValueError:
        return None
    except KeyError:
        return None




